@@ -1321,7 +1321,7 @@ async def read_source_default(
         return ReadSourceResult(source_text=infile.read(), abspath=abspath)
 
 
-async def load_async(
+async def _load_async(
     uri: str,
     path: Optional[List[str]] = None,
     check_quant: bool = True,
@@ -1350,7 +1350,7 @@ async def load_async(
                 imp.pos, imp.uri, "exceeded import_max_depth; circular imports?"
             )
         try:
-            subdoc = await load_async(
+            subdoc = await _load_async(
                 imp.uri,
                 path=path,
                 check_quant=check_quant,
@@ -1367,16 +1367,20 @@ async def load_async(
         doc.typecheck(check_quant=check_quant)
     except Error.ValidationError as exn:
         exn.source_text = read_rslt.source_text
+        exn.declared_wdl_version = doc.wdl_version
         raise exn
     except Error.MultipleValidationErrors as multi:
         for exn in multi.exceptions:
             if not exn.source_text:
                 exn.source_text = read_rslt.source_text
+                exn.declared_wdl_version = doc.wdl_version
+        multi.source_text = read_rslt.source_text
+        multi.declared_wdl_version = doc.wdl_version
         raise multi
     return doc
 
 
-def load(
+def _load(
     uri: str,
     path: Optional[List[str]] = None,
     check_quant: bool = True,
@@ -1387,7 +1391,7 @@ def load(
     importer: Optional[Document] = None,
 ) -> Document:
     return asyncio.get_event_loop().run_until_complete(
-        load_async(
+        _load_async(
             uri,
             path=path,
             importer=importer,
@@ -1404,7 +1408,7 @@ def load(
 
 
 def _calls(element: Union[Workflow, WorkflowSection]) -> Generator[Call, None, None]:
-    # Yield each Call in the workflow, including those nested withis scatter/conditional sections
+    # Yield each Call in the workflow, including those nested within scatter/conditional sections
     for ch in element.children:
         if isinstance(ch, Call):
             yield ch
