@@ -11,7 +11,6 @@ import traceback
 import glob
 import time
 import random
-import multiprocessing
 import threading
 import shutil
 import shlex
@@ -24,6 +23,7 @@ from abc import ABC, abstractmethod
 from typing import Tuple, List, Dict, Optional, Callable, Iterable, Set, Any
 from contextlib import ExitStack
 import docker
+
 from .. import Error, Type, Env, Value, StdLib, Tree, _util
 from .._util import (
     write_atomic,
@@ -814,13 +814,40 @@ def run_local_task(
         write_values_json(inputs, os.path.join(run_dir, "inputs.json"))
 
         if not _run_id_stack:
+            wdl_doc = getattr(task, "parent")
             cache = _cache or cleanup.enter_context(CallCache(cfg, logger))
             cache.flock(logfile, exclusive=True)  # no containing workflow; flock task.log
+
         else:
             cache = _cache
         assert cache
 
         try:
+            input_digest = cache.get_digest_for_inputs(inputs)
+            task_digest = cache.get_digest_for_task(task)
+            cached = cache.get(
+                key=f"{task.name}_{task_digest}/{input_digest}", output_types=task.effective_outputs
+            )
+            if cached:
+                for decl in task.outputs:
+                    v = cached[decl.name]
+                    vj = json.dumps(v.json)
+                    logger.info(
+                        _(
+                            "cached output",
+                            name=decl.name,
+                            value=(v.json if len(vj) < 4096 else "(((large)))"),
+                        )
+                    )
+                # create out/ and outputs.json
+                outputs = link_outputs(
+                    cached, run_dir, hardlinks=cfg["file_io"].get_bool("output_hardlinks")
+                )
+                write_values_json(
+                    cached, os.path.join(run_dir, "outputs.json"), namespace=task.name
+                )
+                logger.notice("done (cached)")  # pyre-fixme
+                return (run_dir, cached)
             # start plugin coroutines and process inputs through them
             with compose_coroutines(
                 [
@@ -892,6 +919,7 @@ def run_local_task(
                     outputs, os.path.join(run_dir, "outputs.json"), namespace=task.name
                 )
                 logger.notice("done")  # pyre-fixme
+                cache.put(f"{task.name}_{task_digest}", input_digest, outputs)
                 return (run_dir, outputs)
         except Exception as exn:
             logger.debug(traceback.format_exc())
@@ -1284,7 +1312,7 @@ def link_outputs(
                 assert os.path.isfile(hardlink)
                 newlink = os.path.join(dn, os.path.basename(v.value))
                 os.makedirs(dn, exist_ok=False)
-                if not hardlinks and path_really_within(hardlink, run_dir):
+                if not hardlinks and path_really_within(hardlink, os.path.dirname(run_dir)):
                     # make symlink relative
                     hardlink = os.path.relpath(hardlink, start=os.path.realpath(dn))
                 (os.link if hardlinks else os.symlink)(hardlink, newlink)
@@ -1323,13 +1351,13 @@ def link_outputs(
                 v.value[key] = map_files(v.value[key], os.path.join(dn, key))
         return v
 
-    os.makedirs(os.path.join(run_dir, "output_links"), exist_ok=False)
+    os.makedirs(os.path.join(run_dir, "out"), exist_ok=False)
+    # out/ used to be called output_links/ -- symlink this name to ease transition
+    os.symlink("out", os.path.join(run_dir, "output_links"))
     return outputs.map(
         lambda binding: Env.Binding(
             binding.name,
-            map_files(
-                copy.deepcopy(binding.value), os.path.join(run_dir, "output_links", binding.name),
-            ),
+            map_files(copy.deepcopy(binding.value), os.path.join(run_dir, "out", binding.name),),
         )
     )
 
