@@ -39,7 +39,6 @@ import signal
 import traceback
 import pickle
 import threading
-import copy
 from concurrent import futures
 from typing import Optional, List, Set, Tuple, NamedTuple, Dict, Union, Iterable, Callable, Any
 from contextlib import ExitStack
@@ -647,7 +646,7 @@ def run_local_workflow(
                 version = "UNKNOWN"
             logger.notice(_("miniwdl", version=version))  # pyre-fixme
             assert not _thread_pools
-
+            wdl_doc = getattr(workflow, "parent")
             cache = _cache or cleanup.enter_context(CallCache(cfg, logger))
             cache.flock(logfile, exclusive=True)  # flock top-level workflow.log
 
