@@ -1,30 +1,34 @@
 import unittest, inspect, subprocess, tempfile, os, glob, json, urllib
 from .context import WDL
 
+
 class Lint(unittest.TestCase):
     # test programmatic access to linter results
     # this code should be kept in sync with the example shown in the WDL/Lint.py docstring.
     def test_api(self):
-        doc = WDL.load(
-            "ZarrUtils.wdl",
-            path=["test_corpi/HumanCellAtlas/skylab/library/tasks"]
-        )
+        doc = WDL.load("ZarrUtils.wdl", path=["test_corpi/HumanCellAtlas/skylab/library/tasks"])
         WDL.Lint._shellcheck_available = False
         lint = WDL.Lint.collect(WDL.Lint.lint(doc, descend_imports=False))
-        for (pos, lint_class, message) in lint:
+        for (pos, lint_class, message, suppressed) in lint:
             assert isinstance(pos, WDL.SourcePosition)
             assert isinstance(lint_class, str) and isinstance(message, str)
-            print(json.dumps({
-                "uri"        : pos.uri,
-                "abspath"    : pos.abspath,
-                "line"       : pos.line,
-                "end_line"   : pos.end_line,
-                "column"     : pos.column,
-                "end_column" : pos.end_column,
-                "lint"       : lint_class,
-                "message"    : message,
-            }))
-        self.assertEqual(len(lint), 1)
+            if not suppressed:
+                print(
+                    json.dumps(
+                        {
+                            "uri": pos.uri,
+                            "abspath": pos.abspath,
+                            "line": pos.line,
+                            "end_line": pos.end_line,
+                            "column": pos.column,
+                            "end_column": pos.end_column,
+                            "lint": lint_class,
+                            "message": message,
+                        }
+                    )
+                )
+        self.assertEqual(len(lint), 2)
+
 
 async def read_source(uri, path, importer_uri):
     if uri.startswith("http:") or uri.startswith("https:"):
@@ -45,8 +49,12 @@ def wdl_corpus(dir, path=[], blacklist=[], expected_lint={}, check_quant=True):
         test_klass.tearDownClass = classmethod(check_lint)
 
         prefix = test_klass.__name__
-        files = glob.glob(os.path.join(*(dir + ['*.wdl'])), recursive=True)
-        assert len(files) > 0, "{} test corpus missing from {}; please `git submodule update --init --recursive`".format(prefix, os.path.join(*dir))
+        files = glob.glob(os.path.join(*(dir + ["*.wdl"])), recursive=True)
+        assert (
+            len(files) > 0
+        ), "{} test corpus missing from {}; please `git submodule update --init --recursive`".format(
+            prefix, os.path.join(*dir)
+        )
         gpath = []
         for p in path:
             gpath = gpath + glob.glob(os.path.join(*p), recursive=True)
@@ -54,13 +62,16 @@ def wdl_corpus(dir, path=[], blacklist=[], expected_lint={}, check_quant=True):
             name = os.path.split(fn)[1]
             name = name[:-4]
             if name not in blacklist:
-                name = "test_" + prefix + "_" + name.replace('.', '_')
+                name = "test_" + prefix + "_" + name.replace(".", "_")
                 while hasattr(test_klass, name):
-                    name += '_'
+                    name += "_"
+
                 def t(self, fn=fn):
                     # load & lint the document to verify the lint count
                     try:
-                        doc = WDL.load(fn, path=gpath, check_quant=check_quant, read_source=read_source)
+                        doc = WDL.load(
+                            fn, path=gpath, check_quant=check_quant, read_source=read_source
+                        )
                     except Exception as exn:
                         if isinstance(exn, WDL.Error.MultipleValidationErrors):
                             for subexn in exn.exceptions:
@@ -69,11 +80,17 @@ def wdl_corpus(dir, path=[], blacklist=[], expected_lint={}, check_quant=True):
                             print(exn.node.pos)
                         raise
                     WDL.Lint.lint(doc)
-                    for _, linter, _ in WDL.Lint.collect(doc):
+                    for _, linter, _, suppressed in WDL.Lint.collect(doc):
                         test_klass._lint_count[linter] = 1 + test_klass._lint_count.get(linter, 0)
+                        if suppressed:
+                            test_klass._lint_count[
+                                "_suppressions"
+                            ] = 1 + test_klass._lint_count.get("_suppressions", 0)
                     print("\n" + os.path.basename(fn))
                     WDL.CLI.outline(doc, 0, show_called=(doc.workflow is not None))
-                    WDL.copy_source(doc, tempfile.mkdtemp(prefix=f"miniwdl_test_copy_source_{prefix}"))
+                    WDL.copy_source(
+                        doc, tempfile.mkdtemp(prefix=f"miniwdl_test_copy_source_{prefix}")
+                    )
 
                     if doc.workflow:
                         validate_workflow_graph(doc.workflow)
@@ -81,23 +98,36 @@ def wdl_corpus(dir, path=[], blacklist=[], expected_lint={}, check_quant=True):
                     # also attempt load with the opposite value of check_quant,
                     # exercising additional code paths
                     try:
-                        doc = WDL.load(fn, path=gpath, check_quant=not check_quant, read_source=read_source)
-                    except (WDL.Error.ImportError, WDL.Error.ValidationError, WDL.Error.MultipleValidationErrors):
+                        doc = WDL.load(
+                            fn, path=gpath, check_quant=not check_quant, read_source=read_source
+                        )
+                    except (
+                        WDL.Error.ImportError,
+                        WDL.Error.ValidationError,
+                        WDL.Error.MultipleValidationErrors,
+                    ):
                         pass
+
                 setattr(test_klass, name, t)
 
         return test_klass
+
     return decorator
 
+
 def check_lint(cls):
     if "CommandShellCheck" in cls._lint_count:
         # because we don't control installed shellcheck version
         del cls._lint_count["CommandShellCheck"]
     if cls._lint_count != cls._expected_lint:
-        raise Exception("Lint results changed for {}; expected: {} got: {}".format(cls.__name__, str(cls._expected_lint), str(cls._lint_count)))
+        raise Exception(
+            "Lint results changed for {}; expected: {} got: {}".format(
+                cls.__name__, str(cls._expected_lint), str(cls._lint_count)
+            )
+        )
+
 
 def validate_workflow_graph(workflow):
-
     def visit_section(body, outside_nodes):
         body_nodes = set()
         body_gathers = set()
@@ -112,50 +142,87 @@ def validate_workflow_graph(workflow):
             unk = node.workflow_node_dependencies - (outside_nodes | body_nodes | body_gathers)
             assert not unk, str((node.workflow_node_id, unk))
             if isinstance(node, WDL.WorkflowSection):
-                visit_section(node.body, (outside_nodes | body_nodes | body_gathers) - set(node.gathers))
+                visit_section(
+                    node.body, (outside_nodes | body_nodes | body_gathers) - set(node.gathers)
+                )
 
     visit_section(workflow.body, set(inp.workflow_node_id for inp in workflow.inputs or []))
 
+
 @wdl_corpus(
     ["test_corpi/HumanCellAtlas/skylab/library/tasks/**"],
-    expected_lint={'UnusedDeclaration': 2, 'UnknownRuntimeKey': 1}
+    expected_lint={"UnusedDeclaration": 2, "UnknownRuntimeKey": 1, "MissingVersion": 26},
 )
 class HCAskylab_task(unittest.TestCase):
     pass
 
+
 @wdl_corpus(
     ["test_corpi/HumanCellAtlas/skylab/pipelines/**"],
     path=[["test_corpi/HumanCellAtlas/skylab/library/tasks"]],
-    expected_lint={'UnusedDeclaration': 12, 'NameCollision': 3, 'UnknownRuntimeKey': 3, 'StringCoercion': 1, 'MixedIndentation': 1, 'FileCoercion': 1}
+    expected_lint={
+        "UnusedDeclaration": 12,
+        "NameCollision": 3,
+        "UnknownRuntimeKey": 3,
+        "StringCoercion": 1,
+        "MixedIndentation": 1,
+        "FileCoercion": 1,
+        "MissingVersion": 34,
+    },
 )
 class HCAskylab_workflow(unittest.TestCase):
     pass
 
+
 @wdl_corpus(
     ["test_corpi/gatk-workflows/five-dollar-genome-analysis-pipeline/**"],
-    expected_lint={'UnusedDeclaration': 5, 'NameCollision': 2, 'UnusedImport': 8},
+    expected_lint={"UnusedDeclaration": 5, "NameCollision": 2, "UnusedImport": 8},
 )
 class GATK_five_dollar(unittest.TestCase):
     pass
 
+
 @wdl_corpus(
     ["test_corpi/gatk-workflows/gatk4-germline-snps-indels/**"],
-    expected_lint={'UnusedDeclaration': 3, 'StringCoercion': 20, 'FileCoercion': 3, 'UnknownRuntimeKey': 1}
+    expected_lint={
+        "UnusedDeclaration": 3,
+        "StringCoercion": 20,
+        "FileCoercion": 3,
+        "UnknownRuntimeKey": 1,
+        "MissingVersion": 4,
+    },
 )
 class gatk4_germline_snps_indels(unittest.TestCase):
     pass
 
+
 @wdl_corpus(
     ["test_corpi/gatk-workflows/gatk4-somatic-snvs-indels/**"],
-    expected_lint={'OptionalCoercion': 50, 'NonemptyCoercion': 4, 'UnusedDeclaration': 29, 'ForwardReference': 6, 'StringCoercion': 20, 'MixedIndentation': 6},
+    expected_lint={
+        "OptionalCoercion": 50,
+        "NonemptyCoercion": 4,
+        "UnusedDeclaration": 29,
+        "ForwardReference": 6,
+        "StringCoercion": 20,
+        "MixedIndentation": 6,
+        "MissingVersion": 6,
+    },
     check_quant=False,
 )
 class gatk4_somatic_snvs_indels(unittest.TestCase):
     pass
 
+
 @wdl_corpus(
     ["test_corpi/gatk-workflows/gatk4-cnn-variant-filter/**"],
-    expected_lint={'UnusedDeclaration': 21, 'OptionalCoercion': 24, 'FileCoercion': 8, 'UnusedCall': 1, 'StringCoercion': 4},
+    expected_lint={
+        "UnusedDeclaration": 21,
+        "OptionalCoercion": 24,
+        "FileCoercion": 8,
+        "UnusedCall": 1,
+        "StringCoercion": 4,
+        "MissingVersion": 6,
+    },
     check_quant=False,
 )
 class gatk4_cnn_variant_filter(unittest.TestCase):
@@ -164,71 +231,131 @@ class gatk4_cnn_variant_filter(unittest.TestCase):
 
 @wdl_corpus(
     ["test_corpi/gatk-workflows/broad-prod-wgs-germline-snps-indels/**"],
-    blacklist=['JointGenotypingWf'],
-    expected_lint={'StringCoercion': 50, 'UnusedDeclaration': 10, 'ArrayCoercion': 4},
-    check_quant=False
+    blacklist=["JointGenotypingWf"],
+    expected_lint={
+        "StringCoercion": 50,
+        "UnusedDeclaration": 10,
+        "ArrayCoercion": 4,
+        "MissingVersion": 2,
+    },
+    check_quant=False,
 )
 class broad_prod_wgs(unittest.TestCase):
     pass
 
+
 @wdl_corpus(
     ["test_corpi/broadinstitute/gtex-pipeline/**"],
     # need URI import
-    blacklist=["rnaseq_pipeline_bam","rnaseq_pipeline_fastq"],
-    expected_lint={'IncompleteCall': 30, 'UnusedDeclaration': 3, 'NameCollision': 4},
+    blacklist=["rnaseq_pipeline_bam", "rnaseq_pipeline_fastq"],
+    expected_lint={
+        "IncompleteCall": 30,
+        "UnusedDeclaration": 3,
+        "NameCollision": 4,
+        "MissingVersion": 23,
+    },
 )
 class GTEx(unittest.TestCase):
     pass
 
+
 @wdl_corpus(
     ["test_corpi/DataBiosphere/topmed-workflows/**"],
     # need URI import
-    blacklist=['CRAM_md5sum_checker_wrapper', 'checker-workflow-wrapping-alignment-workflow',
-                'topmed_freeze3_calling', 'topmed_freeze3_calling_checker', 'u_of_michigan_aligner_checker'],
-    expected_lint={'StringCoercion': 27, 'UnusedDeclaration': 74, 'OptionalCoercion': 1},
+    blacklist=[
+        "CRAM_md5sum_checker_wrapper",
+        "checker-workflow-wrapping-alignment-workflow",
+        "topmed_freeze3_calling",
+        "topmed_freeze3_calling_checker",
+        "u_of_michigan_aligner_checker",
+    ],
+    expected_lint={
+        "StringCoercion": 27,
+        "UnusedDeclaration": 74,
+        "OptionalCoercion": 1,
+        "MissingVersion": 8,
+    },
     check_quant=False,
 )
 class TOPMed(unittest.TestCase):
     pass
 
+
 @wdl_corpus(
     ["test_corpi/broadinstitute/viral-ngs/pipes/WDL/workflows"],
     path=[["test_corpi/broadinstitute/viral-ngs/pipes/WDL/workflows/tasks"]],
-    expected_lint={'UnusedDeclaration': 23, 'NameCollision': 9, 'IncompleteCall': 44, 'UnusedImport': 1, 'SelectArray': 4},
+    expected_lint={
+        "UnusedDeclaration": 23,
+        "NameCollision": 9,
+        "IncompleteCall": 44,
+        "UnusedImport": 1,
+        "SelectArray": 4,
+        "MissingVersion": 62,
+    },
 )
 class ViralNGS(unittest.TestCase):
     pass
 
+
 @wdl_corpus(
     ["test_corpi/ENCODE-DCC/chip-seq-pipeline2/**"],
-    expected_lint={'StringCoercion': 192,  'FileCoercion': 154, 'NameCollision': 16, 'OptionalCoercion': 64, 'MixedIndentation': 32},
+    expected_lint={
+        "StringCoercion": 192,
+        "FileCoercion": 154,
+        "NameCollision": 16,
+        "OptionalCoercion": 64,
+        "MixedIndentation": 32,
+        "MissingVersion": 31,
+    },
     check_quant=False,
 )
 class ENCODE_ChIPseq(unittest.TestCase):
     pass
 
+
 @wdl_corpus(
     ["test_corpi/ENCODE-DCC/atac-seq-pipeline/**"],
-    expected_lint={'UnusedDeclaration': 63, 'MixedIndentation': 15, 'OptionalCoercion': 1020, 'UnusedCall': 45, 'StringCoercion': 30, 'FileCoercion': 71},
+    expected_lint={
+        "UnusedDeclaration": 63,
+        "MixedIndentation": 15,
+        "OptionalCoercion": 1020,
+        "UnusedCall": 45,
+        "StringCoercion": 30,
+        "FileCoercion": 71,
+        "MissingVersion": 29,
+    },
     check_quant=False,
 )
 class ENCODE_ATACseq(unittest.TestCase):
     pass
 
+
 @wdl_corpus(
     ["test_corpi/ENCODE-DCC/rna-seq-pipeline/**"],
-    expected_lint={'StringCoercion': 2, 'UnusedDeclaration': 3, 'IncompleteCall': 3}
+    expected_lint={
+        "StringCoercion": 2,
+        "UnusedDeclaration": 3,
+        "IncompleteCall": 3,
+        "MissingVersion": 3,
+    },
 )
 class ENCODE_RNAseq(unittest.TestCase):
     pass
 
+
 @wdl_corpus(
     ["test_corpi/ENCODE-DCC/wgbs-pipeline/**"],
-    expected_lint={'StringCoercion': 9, 'UnusedDeclaration': 1, 'MixedIndentation': 1}
+    expected_lint={
+        "StringCoercion": 9,
+        "UnusedDeclaration": 1,
+        "MixedIndentation": 1,
+        "MissingVersion": 5,
+    },
 )
 class ENCODE_WGBS(unittest.TestCase):
     pass
 
+
 @wdl_corpus(
     ["test_corpi/dnanexus/dxWDL/test/**"],
     blacklist=[
@@ -237,70 +364,143 @@ class ENCODE_WGBS(unittest.TestCase):
         # decl/output name collision
         "two_levels",
         # use dnanexus extensions
-        "call_native", "call_native_app", "call_native_v1",
+        "call_native",
+        "call_native_app",
+        "call_native_v1",
     ],
     path=[["test_corpi/dnanexus/dxWDL/test/imports/lib"]],
-    expected_lint={'UnusedDeclaration': 34, 'UnusedCall': 16, 'NameCollision': 2, 'OptionalCoercion': 3, 'FileCoercion': 3, 'StringCoercion': 2, 'UnnecessaryQuantifier': 1},
+    expected_lint={
+        "UnusedDeclaration": 34,
+        "UnusedCall": 16,
+        "NameCollision": 2,
+        "OptionalCoercion": 3,
+        "FileCoercion": 3,
+        "StringCoercion": 2,
+        "UnnecessaryQuantifier": 1,
+        "MissingVersion": 52,
+    },
     check_quant=False,
 )
 class dxWDL(unittest.TestCase):
     pass
 
+
 @wdl_corpus(
     ["test_corpi/contrived/**"],
-    expected_lint={'UnusedImport': 4, 'NameCollision': 27, 'StringCoercion': 4, 'FileCoercion': 2, 'NonemptyCoercion': 1, 'UnnecessaryQuantifier': 2, 'UnusedDeclaration': 2, "IncompleteCall": 2, 'SelectArray': 1},
+    expected_lint={
+        "_suppressions": 8,
+        "UnusedImport": 4,
+        "NameCollision": 27,
+        "StringCoercion": 4,
+        "FileCoercion": 2,
+        "NonemptyCoercion": 1,
+        "UnnecessaryQuantifier": 2,
+        "UnusedDeclaration": 2,
+        "IncompleteCall": 2,
+        "SelectArray": 1,
+        "MissingVersion": 7,
+    },
     blacklist=["check_quant", "incomplete_call"],
 )
 class Contrived(unittest.TestCase):
     pass
 
+
 @wdl_corpus(
     ["test_corpi/contrived/**"],
-    expected_lint={'UnusedImport': 6, 'NameCollision': 43, 'StringCoercion': 9, 'FileCoercion': 4, 'OptionalCoercion': 3, 'NonemptyCoercion': 2, 'UnnecessaryQuantifier': 4, 'UnusedDeclaration': 9, 'IncompleteCall': 3, 'ArrayCoercion': 2, 'SelectArray': 4},
+    expected_lint={
+        "_suppressions": 16,
+        "UnusedImport": 6,
+        "NameCollision": 43,
+        "StringCoercion": 9,
+        "FileCoercion": 4,
+        "OptionalCoercion": 3,
+        "NonemptyCoercion": 2,
+        "UnnecessaryQuantifier": 4,
+        "UnusedDeclaration": 9,
+        "IncompleteCall": 3,
+        "ArrayCoercion": 2,
+        "SelectArray": 4,
+        "MissingVersion": 11,
+    },
     check_quant=False,
     blacklist=["incomplete_call"],
 )
 class Contrived2(unittest.TestCase):
     pass
 
+
 @wdl_corpus(
     ["test_corpi/biowdl/tasks/**"],
     blacklist=[
         # these use the pattern 'input { Type? x = default }' and need check_quant=False
-        "mergecounts","somaticseq"
+        "mergecounts",
+        "somaticseq",
     ],
-    expected_lint={'OptionalCoercion': 9, 'UnusedDeclaration': 18, 'NonemptyCoercion': 1, 'NameCollision': 1, 'SelectArray': 1},
+    expected_lint={
+        "OptionalCoercion": 9,
+        "UnusedDeclaration": 18,
+        "NonemptyCoercion": 1,
+        "NameCollision": 1,
+        "SelectArray": 1,
+    },
 )
 class BioWDLTasks(unittest.TestCase):
     pass
 
+
 @wdl_corpus(
     ["test_corpi/biowdl/aligning/**"],
-    expected_lint={'OptionalCoercion': 12, 'UnusedDeclaration': 12, 'NonemptyCoercion': 1, 'NameCollision': 1},
+    expected_lint={
+        "OptionalCoercion": 12,
+        "UnusedDeclaration": 12,
+        "NonemptyCoercion": 1,
+        "NameCollision": 1,
+    },
     check_quant=False,
 )
 class BioWDLAligning(unittest.TestCase):
     pass
 
+
 @wdl_corpus(
     ["test_corpi/biowdl/expression-quantification/**"],
-    expected_lint={'OptionalCoercion': 11, 'UnusedDeclaration': 12, 'NonemptyCoercion': 3, 'NameCollision': 1},
+    expected_lint={
+        "OptionalCoercion": 11,
+        "UnusedDeclaration": 12,
+        "NonemptyCoercion": 3,
+        "NameCollision": 1,
+    },
     check_quant=False,
 )
 class BioWDLExpressionQuantification(unittest.TestCase):
     pass
 
+
 @wdl_corpus(
     ["test_corpi/biowdl/somatic-variantcalling"],
-    expected_lint={'UnusedImport': 2, 'OptionalCoercion': 11, 'UnusedDeclaration': 11, 'NonemptyCoercion': 37, 'SelectArray': 5},
+    expected_lint={
+        "UnusedImport": 2,
+        "OptionalCoercion": 11,
+        "UnusedDeclaration": 11,
+        "NonemptyCoercion": 37,
+        "SelectArray": 5,
+    },
     check_quant=False,
 )
 class BioWDLSomaticVariantCalling(unittest.TestCase):
     pass
 
+
 @wdl_corpus(
     ["test_corpi/biowdl/small-rna"],
-    expected_lint={'UnusedDeclaration': 8, 'SelectArray': 2, 'OptionalCoercion': 2, 'NonemptyCoercion': 3, 'UnusedCall': 1},
+    expected_lint={
+        "UnusedDeclaration": 8,
+        "SelectArray": 2,
+        "OptionalCoercion": 2,
+        "NonemptyCoercion": 3,
+        "UnusedCall": 1,
+    },
     check_quant=False,
 )
 class BioWDLSmallRNA(unittest.TestCase):
